/*  This file is part of hpafs (Hidden Protected Area FileSystem). hpafs is a
 *  FUSE (Filesystem in USErspace).
 *
 *  Copyright (C) 2006, 2007 Paul Bolle <pebolle@tiscali.nl>
 *
 *  hpafs is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  hpafs is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <unistd.h>

#include "dos.h"

int
read_dos(direntry_t * dos, int fd, int debug)
{
    READ_DIRENTRY_MEMBER(flags)
    READ_DIRENTRY_MEMBER(zero0)
    READ_DIRENTRY_MEMBER(service_area_start)
    READ_DIRENTRY_MEMBER(service_area_size)
    READ_DIRENTRY_MEMBER(load_sectors)
    READ_DIRENTRY_MEMBER(load_address)
    READ_DIRENTRY_MEMBER(service_area_id)
    READ_DIRENTRY_MEMBER(id_string)
    READ_DIRENTRY_MEMBER(zero1)
    READ_DIRENTRY_MEMBER(checksum)
    return (1);
}

int
parse_dos(direntry_t * dos, int num) 
{
     int i;
     char id_str[33];

    fprintf(stdout, "dos[%d].flags: %hhd\n", num, dos->flags);
    fprintf(stdout, "    Service area is available as B:      %s\n",
            dos->flags & DOS_DRIVE_B ? "yes" : "");
    fprintf(stdout, "    Diagnostic Service:                  %s\n", 
            dos->flags & DOS_DIAGNOSTIC ? "yes" : "");
    fprintf(stdout, "    Service Area is Read Only:           %s\n",
            dos->flags & DOS_RO ? "yes" : "");
    fprintf(stdout, "    This Boot:                           %s\n",
            dos->flags & DOS_THIS_BOOT ? "yes" : "");
    fprintf(stdout, "    Empty Service Area:                  %s\n",
            dos->flags & DOS_EMPTY ? "yes" : "");
    fprintf(stdout, "    Hidden Service Area:                 %s\n",
            dos->flags & DOS_HIDDEN ? "yes" : "");
    fprintf(stdout, "    Service Area is bootable as A:       %s\n",
            dos->flags & DOS_BOOTABLE_DRIVE_A ? "yes" : "");
    fprintf(stdout, "dos[%d].service_area_start: %lld\n", num,
            dos->service_area_start);
    fprintf(stdout, "dos[%d].service_area_size: %lld\n", num,
            dos->service_area_size);
    fprintf(stdout, "dos[%d].load_sectors: %u\n", num,
            dos->load_sectors);
    fprintf(stdout, "dos[%d].load_address: 0x%04X:%X\n", num,
            (dos->load_address >> 4) & 0xFFFF, dos->load_address & 0xF);
    fprintf(stdout, "dos[%d].service_area_id: %hu\n", num,
            dos->service_area_id);
    for (i = 0; i < 32; i++)
        id_str[i] = dos->id_string[i];
    /* KISS no need to examine the string first, so just attach a NULL */
    id_str[32]= '\0';
    fprintf(stdout, "dos[%d] id_str: %s\n", num, id_str);
    fprintf(stdout, "dos[%d].checksum: %hd\n", num, dos->checksum);
    return (1);
}

int
check_dos(int fd, int debug) 
{
    short tmp, sum = 0;
    int i;

    for (i = 0; i < (DOS_LENGTH/2); i++){
        if (read(fd, &tmp, sizeof(short)) != (ssize_t) 2) {
            if (debug) fprintf(stderr, "check_dos fread short read\n");
            return (0);
        }
        if (debug) fprintf(stderr, "tmp [%d]: %hd\n", i, tmp);
        sum += tmp;
        if (debug) fprintf(stderr, "sum [%d]: %hd\n", i, sum);
    }
    return (sum == 0);
}
