/*  This file is a part of fwfind_hpa. This program is a small utility
 *  program to find the location of "Factory Restore" and "Factory Restore Data"
 *  areas. These areas will be used to D2D factory recover, and the areas are
 * located in HPA(Hidden Protected Area).
 *
 * If you are using old linux kernel, you need to use the SETMAX utility
 * to read HPA. The newer kernel can auto detect and handling the HPA. If
 * you want to check it, see the dmsg of your PC.
 *
 *  This program is using some functions that are part of hpafs
 *  (beer.* and dos.*). The hpafs was programmed by Paul Bolle.
 *
 *
 *  Copyright (C) 2007 Takahiro Adachi <tadachi@tadachi-net.com>
 *
 *  fwfind_hpa is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  fwfind_hpa is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>

#include "beer.h"
#include "dos.h"

#define DefaultDevice           "/dev/hda"
#define SKIP 0

#define FACTORY_RESTORE_ID        260
#define FACTORY_RESTORE_DATA_ID   264

int
search_dos_entry(int dos_num,
		 direntry_t *first_dos,
		 int service_area_id,
		 unsigned long long *start_sector)
{
  int i;
  direntry_t *dos = first_dos;

  for (i=0;i<dos_num;i++) {

    if (dos->service_area_id == service_area_id) {
      *start_sector = dos->service_area_start;
      return 0;
    }
    dos++;
  }
  return 1;
}

int
main(int argc,
     char *argv[])
{
  int fd,i,done = 0;
  beer_t beer;
  direntry_t *dos;
  int offset;
  unsigned long long factory_restore_sec,factory_restore_data_sec;
  char *dev = DefaultDevice;

  if (argc > 1) dev = argv[1];

  fd = open(dev,O_RDONLY);
  lseek(fd, -512, SEEK_END);
  read_beer(&beer, fd, 1);

  if (beer.num_entries == 0) goto err1;

  dos = (direntry_t *) malloc (beer.num_entries * sizeof (direntry_t));
  for (i = 0; i < beer.num_entries; i++) {
    offset = 512 * (1 + (i + 2) / 8) - 64 * ((i + 2 ) % 8);

    lseek(fd, -offset, SEEK_END);
    read_dos(dos+i, fd, 1);
  }

  // "Factory Restore",
  if (search_dos_entry(beer.num_entries,
		       dos,
		       FACTORY_RESTORE_ID,
		       &factory_restore_sec)) goto err2;

  // "Factory Restore Data",
  if (search_dos_entry(beer.num_entries,
		       dos,
		       FACTORY_RESTORE_DATA_ID,
		       &factory_restore_data_sec)) goto err2;
  done = 1;

 err2:
  if (dos) free(dos);
 err1:
  close(fd);

  if (done) {
    printf("%llu %llu",
	   (factory_restore_sec - SKIP)*512,
	   (factory_restore_data_sec - SKIP)*512);
  } else {
    printf("noread");
  }

  return 1-done;
}
